<?php
/**
 * @package   Magic robots.txt
 * @author    ABCdatos
 * @license   GPLv2
 * @link      https://taller.abcdatos.net/robots-txt-wordpress/
*/

defined( 'ABSPATH' ) or die( 'No se permite el acceso.' );

// Determina si contiene un sitemap dentro de la URL de WordPress
function mrt_contiene_sitemap_de_wordpress( $robots_txt_previo ) {
	// No ha de contar con el bloque propio o se lo quita a sí mismo.
	$robots_txt_previo = mrt_retira_robots_propio( $robots_txt_previo );
	$url_wordpress_regex = mrt_url_wordpress_regex();
	$regex = '/^(\b)*sitemap(\b)*:(\s|\t)*' . $url_wordpress_regex . '\w+/im';
	$contiene_sitemap = preg_match( $regex, $robots_txt_previo );
	return $contiene_sitemap;
}

// Regex para la URL de la instalación de WordPress
function mrt_url_wordpress_regex() {
	$url_wordpress = home_url( '/');
	$regex = mrt_convierte_a_regex( $url_wordpress );
	return $regex;
}

/*
// Determina si ya hay un SiteMap
function mrt_contiene_sitemap( $robots_txt_previo ) {
	// La regex es mejorable
	$regex = "/^(\b)*sitemap(\b)*:(\s|\t)*https?:\/\/\w+/im";
	$contiene_sitemap = preg_match( $regex, $robots_txt_previo );
	return $contiene_sitemap;
}
*/

function mrt_sitemap_wp_url() {
	// Ruta del sitemap generado por WP
	if ( mrt_sitemap_yoast_enabled() ) {
		$ruta = mrt_sitemap_yoast_url();
	} else {
		$ruta = mrt_sitemap_core_url();
	}
	return $ruta;
}

function mrt_sitemap_core_url() {
	// Ruta del sitemap generado por el core de WP a partir de la versión 5.5
	$ruta = home_url( '/wp-sitemap.xml' );
	return $ruta;
}

function mrt_sitemap_yoast_url() {
	// Ruta del sitemap generado por Yoast SEO
	$ruta = home_url( '/sitemap_index.xml' );
	return $ruta;
}

function mrt_sitemap_yoast_enabled() {
	// Entrada serializada en option_name wpseo
	// s:18:"enable_xml_sitemap";b:1;
	// s:18:"enable_xml_sitemap";b:0;
	// Yoast SEO > General > Features > XML Sitemaps
	$is_enabled = 0;
	if ( is_plugin_active( 'wordpress-seo' ) ) {
		$yoast_options = get_option( 'wpseo' );
		if ( $yoast_options ) {
			$is_enabled = $yoast_options['enable_xml_sitemap'];
		}
	}
	return $is_enabled;
}

// Basada en wp-includes/sitemaps/class-wp-sitemaps.php
// Plugins como Yoast pueden deshabilitarlo del robots.txt, puede pasar que lo que luego no lo usan en archivo físico y queda huérfano sin ninguno
// falta ver si estando en subdir hace algo
function mrt_sitemaps_enabled() {
	$is_enabled = (bool) get_option( 'blog_public' );
	$is_enabled = (bool) apply_filters( 'wp_sitemaps_enabled', $is_enabled );
	$is_enabled = $is_enabled || mrt_sitemap_yoast_enabled();
	return $is_enabled;
}
